import {vectorTileLayer} from 'esri-leaflet-vector';
import * as geojson from 'geojson';
import * as L from 'leaflet';

import {Location} from '../nshmp-lib/geo';
import {MapBaseLayer} from './map-baselayer.model';
import {VectorId} from './vector-id.model';

/** Location transform */
export type LocationTransform = (location: Location) => Location;

/**
 * Esri raster base layers for Leaflet.
 *
 * https://doc.arcgis.com/en/data-appliance/latest/maps/directory-maps-data.htm
 */
export function baseLayers(): Record<MapBaseLayer, L.LayerGroup> {
  const baseLayers: Record<MapBaseLayer, L.LayerGroup> = {
    // https://maps.arcgis.com/sharing/rest/content/items/33d6280d276d4af89fb66a6ffa7f2072/data
    // https://www.arcgis.com/home/item.html?id=33d6280d276d4af89fb66a6ffa7f2072
    [MapBaseLayer.OCEAN]: L.layerGroup(
      [
        vectorTileLayer(VectorId.OCEAN, {
          ...OPTIONS,
          id: MapBaseLayer.OCEAN,
        }),
        L.tileLayer(arcUrl('Ocean/World_Ocean_Base'), {
          ...OPTIONS,
          id: MapBaseLayer.OCEAN,
        }),
      ],
      {
        attribution: `
          Esri, GEBCO, NOAA, National Geographic, Garmin,
          TomTom, Geonames.org, and other contributors
        `,
      }
    ),

    // https://maps.arcgis.com/sharing/rest/content/items/e128f7554370429c91ef7c0e260c078a/data
    // https://www.arcgis.com/home/item.html?id=e128f7554370429c91ef7c0e260c078a
    //
    [MapBaseLayer.OUTDOOR]: vectorTileLayer(VectorId.OUTDOOR, {
      ...OPTIONS,
      attribution: `
        Esri, TomTom, Garmin, FAO, NOAA, USGS, (c) OpenStreetMap contributors,
        CNES/Airbus DS, InterMap, NASA/METI, NASA/NGS and the GIS User Community
      `,
      id: MapBaseLayer.OUTDOOR,
    }),

    // https://maps.arcgis.com/sharing/rest/content/items/0d17ba68a8f64c278e7e2e7864e1069d/data
    // https://www.arcgis.com/home/item.html?id=0d17ba68a8f64c278e7e2e7864e1069d
    [MapBaseLayer.SATELLITE]: L.layerGroup(
      [
        vectorTileLayer(VectorId.SATELLITE, {
          ...OPTIONS,
          id: MapBaseLayer.SATELLITE,
        }),
        L.tileLayer(arcUrl('World_Imagery'), {
          ...OPTIONS,
          id: MapBaseLayer.SATELLITE,
        }),
      ],
      {
        attribution: 'Esri and its users and partners',
      }
    ),

    // https://maps.arcgis.com/sharing/rest/content/items/22fb75c0fa5a4c88b8ca4c4b8ae5c90b/data
    // https://www.arcgis.com/home/item.html?id=22fb75c0fa5a4c88b8ca4c4b8ae5c90b
    [MapBaseLayer.STREET]: vectorTileLayer(VectorId.STREET, {
      ...OPTIONS,
      attribution: `
        Esri, TomTom, Garmin, FAO, NOAA, USGS, (c) OpenStreetMap contributors,
        and the GIS User Community
      `,
      id: MapBaseLayer.STREET,
    }),

    // https://maps.arcgis.com/sharing/rest/content/items/8d57f09b9df345e79945f0f3da1d4d0f/data
    // https://www.arcgis.com/home/item.html?id=8d57f09b9df345e79945f0f3da1d4d0f
    [MapBaseLayer.TOPO]: vectorTileLayer(VectorId.TOPO, {
      ...OPTIONS,
      attribution: `
        Esri, TomTom, Garmin, FAO, NOAA, USGS, (c) OpenStreetMap contributors,
        CNES/Airbus DS, InterMap, NASA/METI, NASA/NGS and the GIS User Community
      `,
      id: MapBaseLayer.TOPO,
    }),
  };

  return baseLayers;
}

/**
 * Returns a specific Esri raster base layer.
 */
export function baseLayer(baseLayer: MapBaseLayer): L.LayerGroup {
  return baseLayers()[baseLayer];
}

/**
 * Transform coordinates in a Feature Collection.
 *
 * @param featureCollection The feature collection
 * @param transform The transform function to use
 */
export function transformCoordinates(
  featureCollection: geojson.FeatureCollection,
  transform: LocationTransform = loc => loc
): geojson.FeatureCollection {
  for (const feature of featureCollection.features) {
    const geometry = feature.geometry;

    switch (geometry.type) {
      case 'LineString': {
        geometry.coordinates = geometry.coordinates.map(coord =>
          transformCoordinate(coord, transform)
        );
        break;
      }
      case 'MultiLineString': {
        geometry.coordinates = geometry.coordinates.map(position =>
          position.map(coord => transformCoordinate(coord, transform))
        );
        break;
      }
      case 'MultiPoint': {
        geometry.coordinates = geometry.coordinates.map(coord =>
          transformCoordinate(coord, transform)
        );
        break;
      }
      case 'MultiPolygon': {
        geometry.coordinates = geometry.coordinates.map(positions =>
          positions.map(position =>
            position.map(coord => transformCoordinate(coord, transform))
          )
        );
        break;
      }
      case 'Point': {
        geometry.coordinates = transformCoordinate(
          geometry.coordinates,
          transform
        );
        break;
      }
      case 'Polygon': {
        geometry.coordinates = geometry.coordinates.map(position =>
          position.map(coord => transformCoordinate(coord, transform))
        );
        break;
      }
    }
  }

  return featureCollection;
}

const ARCGIS_URL_START =
  'https://services.arcgisonline.com/arcgis/rest/services/';
const ARCGIS_URL_END = '/MapServer/tile/{z}/{y}/{x}';
const OPTIONS: L.MapOptions = {
  maxZoom: 16,
  attributionControl: false,
};

function arcUrl(id: string): string {
  return `${ARCGIS_URL_START}${id}${ARCGIS_URL_END}`;
}

function transformCoordinate(
  coordinates: number[],
  transform: LocationTransform
): number[] {
  const longitude = coordinates[0];
  const latitude = coordinates[1];

  const location = transform({
    latitude,
    longitude,
  });

  return [location.longitude, location.latitude];
}
