import {UncertaintyModel} from '../data';
import {Maths} from './maths.util';

/**
 * No truncation; model ignores truncation level n.
 *
 * Compute the probability of exceeding a value.
 * @param model to compute exceedance
 * @param value for which to compute the exceedance probability
 */
const truncationOff = (model: UncertaintyModel, value: number) => {
  return boundedCcdFn(model, value, 0.0, 1.0);
};

/**
 * No truncation; model ignores truncation level n.
 *
 * Compute the probability of exceeding a value.
 *
 * @param model to compute exceedance
 * @param sequence for which to compute the exceedance probability
 */
const truncationOffSequence = (model: UncertaintyModel, sequence: number[]) => {
  const yValues: number[] = [];

  for (const x of sequence) {
    yValues.push(truncationOff(model, x));
  }

  return yValues;
};

/**
 * Upper truncation only at μ + σ * n.
 *
 * Compute the probability of exceeding a value.
 *
 * @param model to compute exceedance
 * @param value for which to compute the exceedance probability
 */
const truncationUpperOnly = (model: UncertaintyModel, value: number) => {
  return boundedCcdFn(model, value, prob(model), 1.0);
};

/**
 * Upper truncation only at μ + σ * n.
 *
 * Compute the probability of exceeding a value.
 *
 * @param model to compute exceedance
 * @param sequence for which to compute the exceedance probability
 */
const truncationUpperOnlySequence = (
  model: UncertaintyModel,
  sequence: number[]
) => {
  const yValues = [];

  for (const x of sequence) {
    yValues.push(truncationUpperOnly(model, x));
  }

  return yValues;
};

/**
 * Bounded complementary cumulative distribution. Compute the probability that
 * a value will be exceeded, subject to upper and lower probability limits.
 *
 * @param model to compute exceedance
 */
const boundedCcdFn = (
  model: UncertaintyModel,
  value: number,
  pHi: number,
  pLo: number
) => {
  const p = Maths.normalCcdf(model.μ, model.σ, value);
  return probBoundsCheck((p - pHi) / (pLo - pHi));
};

/**
 * For truncated distributions, p may be out of range. For upper truncations,
 * p may be less than pHi, yielding a negative value in boundedCcdFn(); for
 * lower truncations, p may be greater than pLo, yielding a value > 1.0 in
 * boundedCcdFn().
 */
const probBoundsCheck = (p: number) => {
  return p < 0.0 ? 0.0 : p > 1.0 ? 1.0 : p;
};

/**
 * Compute ccd value at μ + nσ.
 *
 * @param model to compute exceedance
 */
const prob = (model: UncertaintyModel) => {
  return Maths.normalCcdf(model.μ, model.σ, model.μ + model.n * model.σ);
};

/**
 * Export functions
 */
export const ExceedanceModel = {
  truncationOff,
  truncationOffSequence,
  truncationUpperOnly,
  truncationUpperOnlySequence,
};
