import {capitalCase, pascalCase} from 'change-case';

/**
 * Identifier for different earthquake {@link Source} types. These are declared
 * in a preferred order of iteration, with potentially longer running source
 * types coming first.
 *
 * Ported from: https://code.usgs.gov/ghsc/nshmp/nshmp-lib
 *    /-/blob/main/src/main/java/gov/usgs/earthquake/nshmp/model/SourceType.java
 */
export enum SourceType {
  /** Décollement (low angle detachment) source. */
  DECOLLEMENT = 'DECOLLEMENT',

  /** Finite fault. */
  FAULT = 'FAULT',

  /** Fault cluster. */
  FAULT_CLUSTER = 'FAULT_CLUSTER',

  /** Fault system. */
  FAULT_SYSTEM = 'FAULT_SYSTEM',

  /** Gridded (background) seismicity. */
  GRID = 'GRID',

  /** Subduction interface. */
  INTERFACE = 'INTERFACE',

  /** Subduction interface cluster. */
  INTERFACE_CLUSTER = 'INTERFACE_CLUSTER',

  /** Subduction interface system. */
  INTERFACE_SYSTEM = 'INTERFACE_SYSTEM',

  /** Subduction interface gridded seismicity. */
  INTERFACE_GRID = 'INTERFACE_GRID',

  /** Subduction intraslab. */
  SLAB = 'SLAB',

  /** Subduction intraslab. */
  SLAB_GRID = 'SLAB_GRID',

  /** The combined source types, used in web services */
  TOTAL = 'TOTAL',

  /** Zone (a.k.a. Area). */
  ZONE = 'ZONE',
}

/**
 * Returns the capital case of the source type string.
 *
 * @param sourceType The source type
 * @deprecated Use `sourceTypeToCapitalCase`
 */
export function sourceTypeToString(sourceType: SourceType): string {
  return capitalCase(sourceType);
}

/**
 * Returns the capital case of the source type string.
 *
 * @param sourceType The source type
 */
export function sourceTypeToCapitalCase(sourceType: SourceType): string {
  return capitalCase(sourceType);
}

/**
 * Returns the pascal case of the source type string.
 *
 * @param sourceType The source type
 */
export function sourceTypeToPascalCase(sourceType: SourceType): string {
  return pascalCase(sourceType);
}

/**
 * Converts source type string into a source type.
 *
 * @param s The capital case of the source type
 */
export function sourceTypeFromCapitalCase(s: string): SourceType {
  const sourceType = Object.values(SourceType).find(
    type => capitalCase(type) === s
  );

  if (sourceType === undefined) {
    throw new Error(`Source type [${s}] does not exists`);
  }

  return sourceType;
}

/**
 * Converts source type string into a source type.
 *
 * @param s The pascal case of the source type
 */
export function sourceTypeFromPascalCase(s: string): SourceType {
  const sourceType = Object.values(SourceType).find(
    type => pascalCase(type) === s
  );

  if (sourceType === undefined) {
    throw new Error(`Source type [${s}] does not exists`);
  }

  return sourceType;
}
